function [ cutsDetected, cuts, diffdata, aboveThresholdArray ] = cut_detection( mov, probePointsXcount, probePointsYcount, diffThreshold, cutThreshold )
%SUMMARY Detect cuts for the input
%Different methods for cut detection are implemented below.
%method = 1:   Intensity histogram difference
%method = 2:   Frame characteristics (see master-thesis)

    nFrames = length(mov);
    [ vHeight, vWidth ] = size(mov(1).cdata(:,:,1));
    cutsDetected = 0;
    cuts = [];
    Fconst = @(x) cutThreshold;
    
    method = 1;
    
    if(method == 1)
        % Compare difference between probepoints in current and last frame
        probePointsXdistance = floor(vWidth / probePointsXcount);
        probePointsYdistance = floor(vHeight / probePointsYcount);

        probePointsX = 1:probePointsXdistance:probePointsXcount*probePointsXdistance;
        probePointsY = 1:probePointsYdistance:probePointsYcount*probePointsYdistance;

        diffdata = zeros(nFrames-1,length(1:3:253));

        color = 1; % Only perform analysis for one color (if there are more than one)
        lastFrame = zeros(probePointsYcount,probePointsXcount,'uint8');
        aboveThresholdArray = zeros(nFrames-1,1);
        for i=2:nFrames
            currentFrame = mov(i).cdata(probePointsY,probePointsX,color);
            diff = abs(currentFrame-lastFrame);
            diffdata(i-1,:) = hist(diff(:),1:3:253);
            % Find how many probe points show a difference greater than the
            % difference threshold
            aboveThreshold = (diff > diffThreshold);
            aboveThresholdArray(i-1) = sum(aboveThreshold(:));
            % If the number of probepoints above the difference threshold
            % exceed the cut threshold, a cut is detected
            if(sum(aboveThreshold(:)) > cutThreshold)
                cutsDetected = cutsDetected + 1;
                cuts(cutsDetected) = i;
            end
            lastFrame = currentFrame;
        end
        
    elseif(method == 2)
        % Compare quadrant signatures for current and last frame
        lastSignature = frame_signature(mov(1).cdata);
        diffdata = zeros(nFrames-1,8);
        aboveThresholdArray = zeros(nFrames-1,1);
        for i=2:nFrames
            % Find the quadrant signiture, which is a summary of
            % intensities in the 4 quadrants:
            currentSignature = frame_signature(mov(i).cdata);
            % Compare the signiture difference
            diff = currentSignature(:) - lastSignature(:);
            diffdata(i-1,:) = diff;
            aboveThresholdArray(i-1) = max(diff(1:4));
            % If the signature difference is larger than the cut threshold,
            % a cut is detected
            if(max(diff(1:4)) > cutThreshold)
                disp(strcat('Cut at frame ',num2str(i)));
                cuts = cuts + 1;
            end
            lastSignature = currentSignature;
        end
        
        figure();
        plot(2:nFrames,aboveThresholdArray,'LineWidth',2);
        hold on;
        plot(2:nFrames,Fconst(2:nFrames),'r','LineWidth',2);
        legend('Frame difference','Threshold');
        hold off;
    else
       disp('Wrong method selected!');        
    end
    
end