%% DESCRIPTION
% This script extracts a random patches of fixed size from a scene and uses
% SIFT-features to detect its presence in other scenes, in order to group
% scenes into scene categories.

%% Initialize environment and load data
close all;
if(~exist('small_mov','var'))
    clear all;
    run('../copy_movie_to_smaller_array.m');
end
load('../cutdetection/bang-youre-dead_cuts.mat'); % Load cuts of the movie
%cuts = cuts(1:11);
run('/home/jan/dtu/speciale/matlab/vlfeat-0.9.18/toolbox/vl_setup');
addpath('../leverageScores');
addpath('..');
warning('off','all');

tic;
%% Set parameters
visualize = 0;
patchWidth = floor(0.3*smallWidth);
patchHeight = floor(0.3*smallHeight);
histogramBins = 10;

%% Helper variables and matrix definitions
sceneCount = length(cuts)-1;
sceneCompareMatrixMin = zeros(sceneCount);
sceneCompareMatrixMean = zeros(sceneCount);
sceneCategoryExperimental = false(sceneCount);

%% Precalculate SIFT-features
% Select random frames from all scenes and calculate the SIFT-features for
% them.
maxScene2FrameCount = 0;
scene2FramePercentage = 0.2;
for scene2=1:sceneCount
    scene2Length = cuts(scene2+1)-cuts(scene2);
    nScene2Frames = ceil(scene2FramePercentage*scene2Length);
    if(nScene2Frames > maxScene2FrameCount)
        maxScene2FrameCount = nScene2Frames;
    end
end
frameSIFTfeatures(1:sceneCount,maxScene2FrameCount) = struct('detector',[],'descriptor',[]);
frameVector = zeros(sceneCount,maxScene2FrameCount);
for scene2=1:sceneCount
    scene2Length = cuts(scene2+1)-cuts(scene2);
    nScene2Frames = ceil(0.2*scene2Length);
    for k=1:nScene2Frames
        % Select frame
        frame = cuts(scene2) + floor(scene2Length*rand);
        Ib = small_mov(frame).cdata;
        frameVector(scene2,k) = frame;
        [ frameSIFTfeatures(scene2,k).detector , frameSIFTfeatures(scene2,k).descriptor ] = vl_sift(single(Ib));
    end
end

%% Iteration over all scenes
for scene1=1:sceneCount
    disp(scene1);
    sceneLength = cuts(scene1+1)-cuts(scene1);
    % Calculate number of patches and how many of these are selected
    nPatches = ceil(10*sceneLength);
    nSampledPatches = ceil(3*sceneLength);
    patches = zeros(patchHeight,patchWidth,nPatches);
    patchSIFTfeatures(1:nSampledPatches) = struct('detector',[],'descriptor',[]);
    variation = zeros(nPatches,1);
    
    patchPositionX = floor((smallWidth-patchWidth)*rand(nPatches,1))+1;
    patchPositionY = floor((smallHeight-patchHeight)*rand(nPatches,1))+1;
    
    % Select candidate patches
    for patch=1:nPatches
        % Select frame
        frame = cuts(scene1) + floor(sceneLength*rand);
        Ia = small_mov(frame).cdata;
        
        patches(:,:,patch) = Ia(patchPositionY(patch):(patchPositionY(patch)+patchHeight-1),...
            patchPositionX(patch):(patchPositionX(patch)+patchWidth-1));
        variation(patch) = std(reshape(patches(:,:,patch),patchWidth*patchHeight,1));
    end
    % Select high-contrast patches
    variation = variation / sum(variation);
    selectedPatches = nonuniform_sampling(variation,nSampledPatches);
    % Calculate SIFT-features for patches
    for j=1:nSampledPatches
        [ patchSIFTfeatures(j).detector, patchSIFTfeatures(j).descriptor ] = vl_sift(single(patches(:,:,selectedPatches(j))));
    end
    % Search for similar patches in other scenes:
    for scene2=1:sceneCount
        scene2Length = cuts(scene2+1)-cuts(scene2);
        nScene2Frames = ceil(scene2FramePercentage*scene2Length);
        matchScores = zeros(nScene2Frames,1);
        for j=1:nScene2Frames  
            frame = frameVector(scene2,j);
            Ib = single(small_mov(frame).cdata);
    
            matchCount = 0;
            matchSum = zeros(nSampledPatches,1);
            for i=1:nSampledPatches
                siftDetectorA = patchSIFTfeatures(i).detector;
                siftDescriptorA = patchSIFTfeatures(i).descriptor;
                siftDetectorB = frameSIFTfeatures(scene2,j).detector;
                siftDescriptorB = frameSIFTfeatures(scene2,j).descriptor;
                
                [ matches, scores ] = vl_ubcmatch(siftDescriptorA,siftDescriptorB);
                displacement = zeros(size(matches,2),2);
                for m=1:size(matches,2)
                    displacement(m,1) = -siftDetectorA(1,matches(1,m)) + siftDetectorB(1,matches(2,m));
                    displacement(m,2) = -siftDetectorA(2,matches(1,m)) + siftDetectorB(2,matches(2,m));
                end
                cPatchStartX = max(1,median(displacement(:,1)));
                cPatchStopX = min(size(Ib,2),median(displacement(:,1))+patchWidth);
                cPatchStartY = max(1,median(displacement(:,2)));
                cPatchStopY = min(size(Ib,1),median(displacement(:,2))+patchHeight);
                correspondingPatch = Ib(cPatchStartY:cPatchStopY,...
                    cPatchStartX:cPatchStopX);
                originalPatch = patches(:,:,selectedPatches(i));
                if(~isempty(displacement))
                    matchSum(i) = sum(abs(hist(originalPatch(:),histogramBins)-hist(correspondingPatch(:),histogramBins)));
                    matchCount = matchCount + 1;
                else
                    matchSum(i) = 0.5*patchWidth*patchHeight;
                end
                %                 if(~isempty(scores))
                %                     matchSum(j) = matchSum(j) + sum(scores); %max(scores) does give worse results
                %                 end
                if(visualize && ~isempty(matches))
                    figure(1);
                    subplot(2,2,1);
                    imagesc(patches(:,:,selectedPatches(i))), colormap(gray), axis image off;
                    vl_plotframe(siftDetectorA(:,matches(1,:)));
                    title('Patch');
                    subplot(2,2,3);
                    scatter(displacement(:,1),displacement(:,2));
                    title('Displacement of SIFT-features');
                    subplot(2,2,[ 2, 4]);
                    imagesc(Ib), colormap(gray),axis image off;
                    hold on;
                    vl_plotframe(siftDetectorB(:,matches(2,:)));
                    if(~isempty(displacement))
                        rectangle('Position',...
                            [median(displacement(:,1)), median(displacement(:,2)),...
                            patchWidth, patchHeight],'EdgeColor','b','LineWidth',2);
                    end
                    hold off;
                    title('Frame');
                    figure(2);
                    subplot(1,2,1);
                    hist(originalPatch(:),histogramBins);
                    subplot(1,2,2);
                    hist(correspondingPatch(:),histogramBins)
                    waitforbuttonpress;
                end
            end
            if(matchCount == 0)
                matchScores(j) = NaN;
            else
                matchScores(j) = nanmin(matchSum);
            end
        end
        sceneCompareMatrixMin(scene1,scene2) = nanmin(matchScores);
        sceneCompareMatrixMean(scene1,scene2) = nanmean(matchScores);
    end
    % Estimate what scenes are related to scene1:
    idxVector = [ 1:scene1-1, scene1+1:sceneCount ];
    meanDiffScene = mean(sceneCompareMatrixMin(scene1,idxVector));
    stdDiffScene = std(sceneCompareMatrixMin(scene1,idxVector));
    categoryCandidates = find(sceneCompareMatrixMin(scene1,:) < meanDiffScene - stdDiffScene);
    sceneCategoryExperimental(scene1,categoryCandidates) = 1;
end
toc;

%% Visualize scene distance for estimated related scenes
minSceneFrameDistanceTrueRelation = sceneCompareMatrixMin(sceneCategoryExperimental);
minSceneFrameDistanceNoRelation = sceneCompareMatrixMin(~sceneCategoryExperimental);
meanSceneFrameDistanceTrueRelation = sceneCompareMatrixMean(sceneCategoryExperimental);
meanSceneFrameDistanceNoRelation = sceneCompareMatrixMean(~sceneCategoryExperimental);

[ minTrueRelationHist, xTrueRelation ] = hist(minSceneFrameDistanceTrueRelation(:),histogramBins);
[ minNoRelationHist, xNoRelation ] = hist(minSceneFrameDistanceNoRelation(:),histogramBins);
reportPlot('plot','Dissimilarity of scenes',[],'Dissimilarity score',[],'Cases');
hold on;
bar(xNoRelation,minNoRelationHist);
bar(xTrueRelation,minTrueRelationHist,'r');
hold off;

[ meanTrueRelationHist, xTrueRelation ] = hist(meanSceneFrameDistanceTrueRelation(:),histogramBins);
[ meanNoRelationHist, xNoRelation ] = hist(meanSceneFrameDistanceNoRelation(:),histogramBins);
reportPlot('plot','Dissimilarity of scenes',[],'Dissimilarity score',[],'Cases');
hold on;
bar(xNoRelation,meanNoRelationHist);
bar(xTrueRelation,meanTrueRelationHist,'r');
hold off;

%% Normalize rows in scene distance matrices
maxVector = sum(sceneCompareMatrixMin,2);
for scene=1:sceneCount
    sceneCompareMatrixMin(scene,:) = sceneCompareMatrixMin(scene,:) / maxVector(scene);
end
maxVector = sum(sceneCompareMatrixMean,2);
for scene=1:sceneCount
    sceneCompareMatrixMean(scene,:) = sceneCompareMatrixMean(scene,:) / maxVector(scene);
end


